@@ -1,6 +1,6 @@
 /*
  * sar/sadc: Report system activity
- * (C) 1999-2008 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2009 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _SA_H
@@ -17,7 +17,7 @@
  */
 
 /* Number of activities */
-#define NR_ACT	23
+#define NR_ACT	30
 
 /* Activities */
 #define A_CPU		1
@@ -43,6 +43,14 @@
 #define A_NET_TCP	21
 #define A_NET_ETCP	22
 #define A_NET_UDP	23
+#define A_NET_SOCK6	24
+#define A_NET_IP6	25
+#define A_NET_EIP6	26
+#define A_NET_ICMP6	27
+#define A_NET_EICMP6	28
+#define A_NET_UDP6	29
+#define A_PWR_CPUFREQ	30
+
 
 /* Macro used to flag an activity that should be collected */
 #define COLLECT_ACTIVITY(m)	act[get_activity_position(act, m)]->options |= AO_COLLECTED
@@ -101,6 +109,10 @@
 #define DISPLAY_CPU_DEF(m)	(((m) & AO_F_CPU_DEF)        == AO_F_CPU_DEF)
 #define DISPLAY_CPU_ALL(m)	(((m) & AO_F_CPU_ALL)        == AO_F_CPU_ALL)
 
+/* Output flags for option -d */
+#define AO_F_DISK_PART		0x00000001
+
+#define COLLECT_PARTITIONS(m)	(((m) & AO_F_DISK_PART)      == AO_F_DISK_PART)
 
 /*
  ***************************************************************************
@@ -126,6 +138,15 @@
 #define K_DISK		"DISK"
 #define K_INT		"INT"
 #define K_SNMP		"SNMP"
+#define K_SOCK6		"SOCK6"
+#define K_IP6		"IP6"
+#define K_EIP6		"EIP6"
+#define K_ICMP6		"ICMP6"
+#define K_EICMP6	"EICMP6"
+#define K_UDP6		"UDP6"
+#define K_IPV6		"IPV6"
+#define K_POWER		"POWER"
+#define K_XDISK		"XDISK"
 
 /* sadc program */
 #define SADC		"sadc"
@@ -234,6 +255,21 @@
 
 #define _buf0	buf[0]
 
+/* Structure used to define a bitmap needed by an activity */
+struct act_bitmap {
+	/*
+	 * Bitmap for activities that need one. Remember to allocate it
+	 * before use!
+	 */
+	unsigned char *b_array;
+	/*
+	 * Size of the bitmap in bits. In fact, bitmap is sized to bitmap_size + 1
+	 * to take into account CPU "all"
+	 */
+	int b_size;
+};
+
+/* Structure used to define an activity */
 struct activity {
 	/*
 	 * This variable contains the identification value (A_...) for this activity.
@@ -317,15 +353,10 @@ struct activity {
 	 */
 	void *buf[3];
 	/*
-	 * Bitmap for activities that need one. Such a bitmap is needed by activity if
-	 * @bitmap_size has a non zero value. Bitmap should be allocated before use!
+	 * Bitmap for activities that need one. Such a bitmap is needed by activity
+	 * if @bitmap is not NULL.
 	 */
-	unsigned char *bitmap;
-	/*
-	 * Size of the bitmap in bits. In fact, bitmap is sized to bitmap_size + 1
-	 * to take into account CPU "all"
-	 */
-	int bitmap_size;
+	struct act_bitmap *bitmap;
 };
 
 
@@ -541,38 +572,6 @@ struct record_header {
  ***************************************************************************
  */
 
-/*
- * Structure used to compute statistics average
- * for counters which are not cumulative.
- */
-struct stats_sum {
-	unsigned long long frmkb;
-	unsigned long long bufkb;
-	unsigned long long camkb;
-	unsigned long long frskb;
-	unsigned long long tlskb;
-	unsigned long long caskb;
-	unsigned long long comkb;
-	unsigned long long dentry_stat;
-	unsigned long long file_used;
-	unsigned long long inode_used;
-	unsigned long long pty_nr;
-	unsigned long long sock_inuse;
-	unsigned long long tcp_inuse;
-	unsigned long long tcp_tw;
-	unsigned long long udp_inuse;
-	unsigned long long raw_inuse;
-	unsigned long long frag_inuse;
-	unsigned long long nr_running;
-	unsigned long long nr_threads;
-	unsigned long load_avg_1;
-	unsigned long load_avg_5;
-	unsigned long load_avg_15;
-	unsigned long count;
-};
-
-#define STATS_SUM_SIZE	(sizeof(struct stats_sum))
-
 /* Structure for timestamps */
 struct tstamp {
 	int tm_sec;
@@ -647,6 +646,20 @@ extern __read_funct_t
 	wrap_read_net_etcp(struct activity *);
 extern __read_funct_t
 	wrap_read_net_udp(struct activity *);
+extern __read_funct_t
+	wrap_read_net_sock6(struct activity *);
+extern __read_funct_t
+	wrap_read_net_ip6(struct activity *);
+extern __read_funct_t
+	wrap_read_net_eip6(struct activity *);
+extern __read_funct_t
+	wrap_read_net_icmp6(struct activity *);
+extern __read_funct_t
+	wrap_read_net_eicmp6(struct activity *);
+extern __read_funct_t
+	wrap_read_net_udp6(struct activity *);
+extern __read_funct_t
+	wrap_read_cpuinfo(struct activity *);
 
 /* Other functions */
 extern void
@@ -680,6 +693,8 @@ extern int
 	get_activity_position(struct activity * [], unsigned int);
 extern char *
 	get_devname(unsigned int, unsigned int, int);
+extern void
+	get_file_timestamp_struct(unsigned int, struct tm *, struct file_header *);
 extern void
 	get_itv_value(struct record_header *, struct record_header *,
 		      unsigned int, unsigned long long *, unsigned long long *);
